'''
@author: JJZHK
@license: (C) Copyright 2017-2023
@contact: jeffcobile@gmail.com
@Software : PyCharm
@file: aaa.py
@time: 2018/10/25 14:11
@desc:
'''
import numpy as np
import PIL.ImageDraw as pil_imagedraw
import PIL.ImageFont as pil_imagefont
import jjzhk.imageutils as eui
import cv2
import os


class DrawDetectionMask:
    def __init__(self, class_name = dict(), output=str):
        self.class_name = class_name
        self.class_color = [
            [184, 255, 0], [31, 0, 255], [0, 159, 255], [255, 95, 0], [255, 19, 0], [255, 0, 0],
            [255, 38, 0], [0, 255, 25], [255, 0, 133], [255, 172, 0], [108, 0, 255], [0, 82, 255],
            [0, 255, 6], [255, 0, 152], [223, 0, 255], [12, 0, 255], [0, 255, 178], [108, 255, 0],
            [184, 0, 255], [255, 0, 76], [146, 255, 0], [51, 0, 255], [0, 197, 255], [255, 248, 0],
            [255, 0, 19], [255, 0, 38], [89, 255, 0], [127, 255, 0], [255, 153, 0], [0, 255, 255],
            [0, 255, 216], [0, 255, 121], [255, 0, 248], [70, 0, 255], [0, 255, 159],
            [0, 216, 255], [0, 6, 255], [0, 63, 255], [31, 255, 0], [255, 57, 0],
            [255, 0, 210], [0, 255, 102], [242, 255, 0], [255, 191, 0], [0, 255, 63],
            [255, 0, 95], [146, 0, 255], [184, 255, 0], [255, 114, 0], [0, 255, 235], [255, 229, 0],
            [0, 178, 255], [255, 0, 114], [255, 0, 57], [0, 140, 255], [0, 121, 255], [12, 255, 0],
            [255, 210, 0], [0, 255, 44], [165, 255, 0], [0, 25, 255], [0, 255, 140], [0, 101, 255],
            [0, 255, 82], [223, 255, 0], [242, 0, 255], [89, 0, 255], [165, 0, 255], [70, 255, 0],
            [255, 0, 172], [255, 76, 0], [203, 255, 0], [204, 0, 255], [255, 0, 229], [255, 133, 0],
            [127, 0, 255], [0, 235, 255], [0, 255, 197], [255, 0, 191], [0, 44, 255], [50, 255, 0]
        ]
        self.output = output

    def __call__(self, imagepath, boxes, mask, **kwargs):
        image = cv2.imread(imagepath)
        cv2topil = eui.CV2ToPIL()
        img = cv2topil(image)
        fileinfo = str(imagepath).split('/')
        imagename = fileinfo[len(fileinfo) - 1].split('.')[0]
        img, mask = self._draw_box_(img, boxes, mask)

        img.save(os.path.join(self.output, "%s.jpg" % imagename))
        if mask is not None:
            mask.save(os.path.join(self.output, "%s_mask.jpg" % imagename))

    def _draw_box_(self, image, boxes, mask):
        font = pil_imagefont.truetype(font='Courier.dfont',
                                      size=np.floor(3e-2 * image.size[1] + 0.5).astype('int32'))
        thickness = (image.size[0] + image.size[1]) // 300
        if boxes is not None:
            for (x1, y1), (x2, y2), class_name, _, prob, boxcolor in boxes:
                draw = pil_imagedraw.Draw(image)
                left = int(x1)
                top = int(y1)
                right = int(x2)
                bottom = int(y2)
                color = tuple(boxcolor)

                label = '{} {:.2f}'.format(class_name, prob)
                label_size = draw.textsize(label, font)

                for i in range(thickness):
                    draw.rectangle(
                        [left + i, top + i, right - i, bottom - i], outline=color)

                if top - label_size[1] >= 0:
                    text_origin = np.array([left, top - label_size[1]])
                else:
                    text_origin = np.array([left, top + 1])

                draw.rectangle(
                    [tuple(text_origin), tuple(text_origin + label_size)],
                    fill=color)
                draw.text(text_origin, label, fill=(0, 0, 0), font=font)
                del draw

        if mask is not None:

            mask = cv2.cvtColor(self._maskToImg_(mask, self.class_name, self.class_color), cv2.COLOR_RGB2BGR)
            piltocv2 = eui.PILToCV2()
            image = piltocv2(image)
            image = cv2.addWeighted(image, 0.6, mask, 0.4, 0)
            cv2topil = eui.CV2ToPIL()
            image = cv2topil(image)
            mask = cv2topil(mask)
        return image, mask

    def _maskToImg_(self, mask, CLASS, COLOR):
        r = mask.copy()
        g = mask.copy()
        b = mask.copy()
        for ll in range(1, len(CLASS) + 1):
            r[mask == ll] = COLOR[ll][0]
            g[mask == ll] = COLOR[ll][1]
            b[mask == ll] = COLOR[ll][2]
        rgb = np.zeros((mask.shape[0], mask.shape[1], 3), dtype=np.uint8)
        rgb[:, :, 0] = r
        rgb[:, :, 1] = g
        rgb[:, :, 2] = b

        return rgb